#include "functions.h"

int main() {
    int n =3;
    int m =3;
	gsl_matrix* A = gsl_matrix_alloc(n,m);
    gsl_matrix* B = gsl_matrix_alloc(m,n);
    gsl_matrix* AB = gsl_matrix_alloc(n,n);
    gsl_matrix* Q = gsl_matrix_alloc(n,m);
    gsl_matrix* R = gsl_matrix_alloc(m,m);
    gsl_matrix* QR = gsl_matrix_alloc(n,m);
    gsl_matrix* I = gsl_matrix_alloc(m,m);
    gsl_vector* x = gsl_vector_alloc(n);
	gsl_vector* b = gsl_vector_alloc(n);
    gsl_vector* y = gsl_vector_alloc(n);
    gsl_matrix_set(A,0,0,3.13);
	gsl_matrix_set(A,0,1,8.08);
	gsl_matrix_set(A,0,2,-4.36);
	gsl_matrix_set(A,1,0,-2.90);
	gsl_matrix_set(A,1,1,-6.31);
	gsl_matrix_set(A,1,2,1.00);
	gsl_matrix_set(A,2,0,5.86);
	gsl_matrix_set(A,2,1,-3.89);
	gsl_matrix_set(A,2,2,0.19);
    /*
    gsl_matrix_set(A,3,0,2.76);
	gsl_matrix_set(A,3,1,-5.21);
	gsl_matrix_set(A,3,2,1.59);
    gsl_matrix_set(A,4,0,7.68);
	gsl_matrix_set(A,4,1,4.20);
	gsl_matrix_set(A,4,2,-6.31);
    */
    gsl_vector_set(b,0,6.23);
	gsl_vector_set(b,1,5.37);
	gsl_vector_set(b,2,2.29);
    gsl_matrix_memcpy(Q,A);

    printf("\n--------------------\nGS-decomp algorithm test:\n\n");
    printf("Before GS-decomp:\nA matrix:\n");
    print_matrix(A);
    GS_decomp(Q,R);
    printf("After GS-decomp:\nQ matrix:\n");
    print_matrix(Q);
    printf("R matrix:\n");
    print_matrix(R);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,Q,R,0,QR);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,Q,Q,0,I);
    printf("QR product:\n");
    print_matrix(QR);
    printf("Q transposed times Q:\n");
    print_matrix(I);

    printf("\n\n--------------------\nGS_solve algorith test\n\n");

    GS_solve(Q,R,b,x);
    gsl_blas_dgemv(CblasNoTrans,1,A,x,0,y);
    print_vector("Solution x to A*x=b:",x);
    print_vector("b:",b);
    print_vector("A*x:",y);

    printf("\n--------------------\nGS_inverse algorith test\n\n");

    GS_inverse(Q,R,B);
    printf("A's inverse:\n");
    print_matrix(B);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,A,B,0,AB);
    printf("A*B:\n");
    print_matrix(AB);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,B,A,0,AB);
    printf("B*A:\n");
    print_matrix(AB);

    FILE* time_test_out_stream = fopen("time_test.txt","w");
    for(int N=250;N<500;N+=2) {
        int iterations = 3;
        double my_time = 0;
        double gsl_time = 0;
        for(int i=0;i<iterations;i++) {
            gsl_matrix* My_time_test_A = gsl_matrix_alloc(N,N);
            gsl_matrix* Gsl_time_test_A = gsl_matrix_alloc(N,N);
            gsl_matrix* My_time_test_R = gsl_matrix_alloc(N,N);
            gsl_vector* Gsl_time_test_v = gsl_vector_alloc(N);

            for(int k=0; k<My_time_test_A->size1;k++) {
                for(int j=0; j<My_time_test_A->size2;j++) {
                    double kj=RND;
                    gsl_matrix_set(My_time_test_A,k,j,kj);
                }
            }
            gsl_matrix_memcpy(Gsl_time_test_A,My_time_test_A);

            clock_t my_time1 = clock();
            GS_decomp(My_time_test_A,My_time_test_R);
            clock_t my_time2 = clock();
            my_time += (double)(my_time2-my_time1);

            clock_t gsl_time1 = clock();
            gsl_linalg_QR_decomp(Gsl_time_test_A,Gsl_time_test_v);
            clock_t gsl_time2 = clock();
            gsl_time += (double)(gsl_time2-gsl_time1);

            gsl_matrix_free(My_time_test_A);
            gsl_matrix_free(Gsl_time_test_A);
            gsl_matrix_free(My_time_test_R);
            gsl_vector_free(Gsl_time_test_v);
        }
        my_time /= iterations; my_time /= CLOCKS_PER_SEC;
        gsl_time /= iterations; gsl_time /= CLOCKS_PER_SEC;
        double func_val = 0.001+1.6e-9*pow(N,3);
        fprintf(time_test_out_stream,"%i %g %g %g\n",N,my_time,gsl_time,func_val);
    }
    fclose(time_test_out_stream);

    gsl_matrix_free(A);
    gsl_matrix_free(B);
    gsl_matrix_free(AB);
    gsl_matrix_free(Q);
    gsl_matrix_free(R);
    gsl_matrix_free(QR);
    gsl_matrix_free(I);
    gsl_vector_free(x);
	gsl_vector_free(b);
    gsl_vector_free(y);
return 0;
}