#include "functions.h"
#define nr 2*64

static double xs[nr]; static double ys[nr];
static double my_cubic_b[nr-1];
static double my_cubic_c[nr-1];
static double my_cubic_d[nr-1];
static cubic_coefs coefs = {.b=my_cubic_b,.c=my_cubic_c,.d=my_cubic_d};

void aux(gsl_vector* x, gsl_vector* fx) {
    double energy = gsl_vector_get(x,0);
    double M = cubic_eval(coefs,nr,xs,ys,energy);
    gsl_vector_set(fx,0,M);
}

int main() {
    printf("Finding root of Rosenbrock's valley function:\n");
    gsl_vector* x0 = gsl_vector_alloc(2);
    gsl_vector_set(x0,0,-3);
    gsl_vector_set(x0,1,3);
    print_vector("Starting guess:",x0);
    newton_root(f,x0,1e-9);
    print_vector("Found root:",x0);
    gsl_vector_free(x0);

    printf("\n\nFinding bound states of hydrogen with simple boundary:\n");
    //Making spline of Fe(rmax)(e) since its instability makes newtons method inable to calculate proper derivates (See func.png)
    for(int rmax=2;rmax<=10;rmax+=2) {
        int index = 0;
        for(double e=-2;e<0;e+=2.0/(nr)) {xs[index]=e;ys[index]=Fe_simple(e,rmax,"N");index+=1;}
        cubic_interpol(nr,xs,ys,&coefs);
        gsl_vector* x00 = gsl_vector_alloc(1);
        double guess = -1.0;
        gsl_vector_set(x00,0,guess);
        printf("Rmax = %i\n",rmax);
        print_vector("Starting guess:",x00);
        newton_root(aux,x00,1e-9);
        print_vector("Found root:",x00);
        double energy = gsl_vector_get(x00,0);
        char filename[30];
        sprintf(filename,"simplepath%i.txt",rmax);
        Fe_simple(energy,rmax,filename);
        gsl_vector_free(x00);
    }
    printf("\n\nFinding bound states of hydrogen with improved boundary:\n");
    for(int rmax=1;rmax<=4;rmax++) {
        int index = 0;
        double rd = (double)rmax;
        for(double e=-2;e<0;e+=2.0/(nr)) {xs[index]=e;ys[index]=Fe_simple(e,rmax,"N")-rd*exp(-rd*sqrt(-2*e));index+=1;}
        cubic_interpol(nr,xs,ys,&coefs);
        gsl_vector* x00 = gsl_vector_alloc(1);
        double guess = -1.0;
        gsl_vector_set(x00,0,guess);
        printf("Rmax = %i\n",rmax);
        print_vector("Starting guess:",x00);
        newton_root(aux,x00,1e-9);
        print_vector("Found root:",x00);
        double energy = gsl_vector_get(x00,0);
        char filename[30];
        sprintf(filename,"improvedpath%i.txt",rmax);
        Fe_simple(energy,rmax,filename);
        gsl_vector_free(x00);
    }
    FILE* exact_stream = fopen("exact.txt","w");
    for(double r=1e-3;r<8;r+=1.0/64) {
        fprintf(exact_stream,"%g %g\n",r,r*exp(-r));
    }
    fclose(exact_stream);
    FILE* plot_stream = fopen("Fe.txt","w");
    for(double r=-0.35;r<-0.32;r+=1.0e-4) {
        fprintf(plot_stream,"%g %g\n",r,Fe_simple(r,8,"N"));
    }
    fclose(plot_stream);
return 0;
}